// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.command;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.NotEnabledException;
import org.eclipse.core.commands.NotHandledException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.CompoundContributionItem;
import org.eclipse.ui.handlers.IHandlerService;

import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Dec 13, 2010
 * @version 1.0
 */
public class DynamicMarkerTypeEditingMenu extends CompoundContributionItem {
	
	private IHandlerService handlerService = 
		(IHandlerService) PlatformUI.getWorkbench().getService(IHandlerService.class);
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.actions.CompoundContributionItem#getContributionItems()
	 */
	@Override
	protected IContributionItem[] getContributionItems() {

		return new IContributionItem[] {
				new ContributionItem() {
					public void fill(Menu menu, int index){
						for(MarkerType markerType : CdmStore.getTermManager().getPreferredTerms(MarkerType.class)){
							createMenuItem(menu, markerType);
						}
					}
				}
		};
	}
	
	private void createMenuItem(Menu menu, final MarkerType markerType) {

		MenuItem subMenuItem = new MenuItem(menu, SWT.CASCADE);
		subMenuItem.setText(String.format("Set Flag '%s'", markerType.getLabel()));
		
		Menu subMenu = new Menu(menu.getShell(), SWT.DROP_DOWN);
		subMenuItem.setMenu(subMenu);
		
		MenuItem trueItem = new MenuItem(subMenu, SWT.PUSH);
		trueItem.setText("Yes");
		trueItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				doSetFlagCommand(markerType, true);
			}	
		});
		
		MenuItem falseItem = new MenuItem(subMenu, SWT.PUSH);
		falseItem.setText("No");
		falseItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				doSetFlagCommand(markerType, false);
			}						
		});
	}
	
	private void doSetFlagCommand(MarkerType markerType, boolean flag) {
		try {
			Event event = new Event();
			event.data = new Object[]{markerType, flag};
			handlerService.executeCommand("taxeditor-bulkeditor.command.setMarkerFlag", event);
		} catch (ExecutionException e) {
			MessagingUtils.error(getClass(), e);
		} catch (NotDefinedException e) {
			MessagingUtils.error(getClass(), e);
		} catch (NotEnabledException e) {
			MessagingUtils.error(getClass(), e);
		} catch (NotHandledException e) {
			MessagingUtils.error(getClass(), e);
		}
		
	}		

}
