// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.handler;

import java.util.Iterator;
import java.util.Set;
import java.util.UUID;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.IWorkbenchPage;

import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.IGroupService;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.api.service.config.NameDeletionConfigurator;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.persistence.dao.hibernate.HibernateBeanInitializer;
import eu.etaxonomy.cdm.api.application.ICdmApplicationConfiguration;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotation;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotationModel;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.bulkeditor.internal.TaxeditorBulkeditorPlugin;
import eu.etaxonomy.taxeditor.bulkeditor.referencingobjects.ReferencingObjectsView;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;


/**
 * @author n.hoffmann
 * @created Mar 11, 2011
 * @version 1.0
 */
public class DeleteHandler extends AbstractHandler {

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		ISelection selection = HandlerUtil.getCurrentSelection(event);
		
		IEditorPart editor = HandlerUtil.getActiveEditor(event);
		
		IEditorInput input = editor.getEditorInput();
		
		if((input instanceof IEntityPersistenceService) && (selection instanceof IStructuredSelection)){
			
			
			IDocumentProvider provider = ((BulkEditor) editor).getDocumentProvider();
			LineAnnotationModel model = 
					(LineAnnotationModel) provider.getAnnotationModel(input);
			
			
			IStructuredSelection structuredSelection = (IStructuredSelection) selection;
			
			IEntityPersistenceService persistenceService = (IEntityPersistenceService) input;
			
			
			for(Object object : structuredSelection.toList()){
				
				LineAnnotation annotation = (LineAnnotation) model.getAnnotation(object);
				DeleteResult result = new DeleteResult();	
				//result.setError();
				try {
					ICdmApplicationConfiguration controller;
					controller = (ICdmApplicationConfiguration) CdmStore.getCurrentApplicationConfiguration();
					//persistenceService.delete(object);
					if (object instanceof SpecimenOrObservationBase){
						
						IOccurrenceService service = controller.getOccurrenceService();
						SpecimenOrObservationBase persistedObject = service.load(((SpecimenOrObservationBase) object).getUuid());
						if (persistedObject != null){
							result = service.delete(persistedObject);
						} 
					} else if (object instanceof Reference){
						IReferenceService service = controller.getReferenceService();
						
						Reference persistedObject = service.load(((Reference) object).getUuid());
						if (persistedObject != null){
							result = service.delete(persistedObject);
						}
						
					} else if (object instanceof Group){
						IGroupService service = controller.getGroupService();
						Group persistedObject = service.load(((Group) object).getUuid());
						if (persistedObject != null){
							result = service.delete(persistedObject);
						}
					}else if (object instanceof User){
						IUserService service = controller.getUserService();
						User persistedObject = service.load(((User) object).getUuid());
						if (persistedObject != null){
							result = service.delete(persistedObject);
						}
					} else if (object instanceof TaxonNameBase){
						INameService service = controller.getNameService();
						TaxonNameBase name = service.load(((TaxonNameBase) object).getUuid());
						if (name != null){
							NameDeletionConfigurator config = new NameDeletionConfigurator();
							
							result = service.delete(name.getUuid(), config);
						} 
					} else if (object instanceof TaxonBase){
						ITaxonService service = controller.getTaxonService();
						TaxonBase taxonBase = service.load(((TaxonBase) object).getUuid());
						if (taxonBase != null){
							if (object instanceof Taxon){
								result = service.deleteTaxon(((TaxonBase) object).getUuid(), null, null);
							}else{
								result = service.deleteSynonym((Synonym)object, null);
							}
						}
					} else if (object instanceof TeamOrPersonBase){
						IAgentService service = controller.getAgentService();
						TeamOrPersonBase teamOrPerson = (TeamOrPersonBase)service.load(((TeamOrPersonBase) object).getUuid());
						result = service.delete((TeamOrPersonBase)object);
					}
				} catch (Exception e){
					MessagingUtils.errorDialog("Exception occured. Delete not possible", getClass(), e.getMessage(), TaxeditorBulkeditorPlugin.PLUGIN_ID, null, true);
				}
				if (result.isError() || result.isAbort()){
					if (!result.getExceptions().isEmpty()) {
						String message = null;
						int i = result.getExceptions().size();
						for (Exception e:result.getExceptions()){
							i--;
							message+= e.getMessage();
							if (i>0){
								message+= ", ";
							}
						}
						
						
						MessagingUtils.messageDialog("Delete not possible", getClass(), result.getExceptions().toString(), null);
					}else{
						MessagingUtils.messageDialog("Delete not possible", getClass(), "The object could not be deleted. An exception occured.", null);
						
					}
				}
				if (result.isOk() ){
					((BulkEditor) editor).removeAnnotatedLine(annotation);
					if(result.getUpdatedObjects().size() != 0 || !result.getExceptions().isEmpty()){
						MessagingUtils.informationDialog("Delete successfull", "The object is deleted but there are updated objects: " + result.toString());
					}

				}
				
			}				
		}
	
		
		return null;
	}

}
